@@ -100,9 +100,6 @@ type snapshot struct {
 	// It may invalidated when a file's content changes.
 	files filesMap
 
-	// parseCache holds an LRU cache of recently parsed files.
-	parseCache *parseCache
-
 	// symbolizeHandles maps each file URI to a handle for the future
 	// result of computing the symbols declared in that file.
 	symbolizeHandles *persistent.Map // from span.URI to *memoize.Promise[symbolizeResult]
@@ -187,16 +184,6 @@ type snapshot struct {
 	// detect ignored files.
 	ignoreFilterOnce sync.Once
 	ignoreFilter     *ignoreFilter
-
-	// If non-nil, the result of computing orphaned file diagnostics.
-	//
-	// Only the field, not the map itself, is guarded by the mutex. The map must
-	// not be mutated.
-	//
-	// Used to save work across diagnostics+code action passes.
-	// TODO(rfindley): refactor all of this so there's no need to re-evaluate
-	// diagnostics during code-action.
-	orphanedFileDiagnostics map[span.URI]*source.Diagnostic
 }
 
 var globalSnapshotID uint64
@@ -660,7 +647,7 @@ func (s *snapshot) PackageDiagnostics(ctx context.Context, ids ...PackageID) (ma
 		return true
 	}
 	post := func(_ int, pkg *Package) {
-		collect(pkg.ph.m.Diagnostics)
+		collect(pkg.m.Diagnostics)
 		collect(pkg.pkg.diagnostics)
 	}
 	return perFile, s.forEachPackage(ctx, ids, pre, post)
@@ -682,7 +669,7 @@ func (s *snapshot) References(ctx context.Context, ids ...PackageID) ([]source.X
 		return true
 	}
 	post := func(i int, pkg *Package) {
-		indexes[i] = XrefIndex{m: pkg.ph.m, data: pkg.pkg.xrefs()}
+		indexes[i] = XrefIndex{m: pkg.m, data: pkg.pkg.xrefs()}
 	}
 	return indexes, s.forEachPackage(ctx, ids, pre, post)
 }
@@ -867,26 +854,21 @@ func (s *snapshot) getActivePackage(id PackageID) *Package {
 	return nil
 }
 
-// memoizeActivePackage checks if pkg is active, and if so either records it in
+// setActivePackage checks if pkg is active, and if so either records it in
 // the active packages map or returns the existing memoized active package for id.
-//
-// The resulting package is non-nil if and only if the specified package is open.
-func (s *snapshot) memoizeActivePackage(id PackageID, pkg *Package) (active *Package) {
+func (s *snapshot) setActivePackage(id PackageID, pkg *Package) {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
-	if value, ok := s.activePackages.Get(id); ok {
-		return value.(*Package) // possibly nil, if we have already checked this id.
+	if _, ok := s.activePackages.Get(id); ok {
+		return // already memoized
 	}
 
-	defer func() {
-		s.activePackages.Set(id, active, nil) // store the result either way: remember that pkg is not open
-	}()
-
 	if containsOpenFileLocked(s, pkg.Metadata()) {
-		return pkg
+		s.activePackages.Set(id, pkg, nil)
+	} else {
+		s.activePackages.Set(id, (*Package)(nil), nil) // remember that pkg is not open
 	}
-	return nil
 }
 
 func (s *snapshot) resetActivePackagesLocked() {
@@ -1268,17 +1250,6 @@ func (s *snapshot) clearShouldLoad(scopes ...loadScope) {
 	}
 }
 
-// noRealPackagesForURILocked reports whether there are any
-// non-command-line-arguments packages containing the given URI.
-func (s *snapshot) noRealPackagesForURILocked(uri span.URI) bool {
-	for _, id := range s.meta.ids[uri] {
-		if !source.IsCommandLineArguments(id) || s.meta.metadata[id].Standalone {
-			return false
-		}
-	}
-	return true
-}
-
 func (s *snapshot) FindFile(uri span.URI) source.FileHandle {
 	s.view.markKnown(uri)
 
@@ -1364,11 +1335,6 @@ func (s *snapshot) IsOpen(uri span.URI) bool {
 	return open
 }
 
-func isFileOpen(fh source.FileHandle) bool {
-	_, open := fh.(*Overlay)
-	return open
-}
-
 // TODO(rfindley): it would make sense for awaitLoaded to return metadata.
 func (s *snapshot) awaitLoaded(ctx context.Context) error {
 	loadErr := s.awaitLoadedAllErrors(ctx)
@@ -1632,10 +1598,6 @@ func (s *snapshot) reloadOrphanedOpenFiles(ctx context.Context) error {
 	for _, file := range files {
 		file := file
 		g.Go(func() error {
-			pgf, err := s.ParseGo(ctx, file, source.ParseHeader)
-			if err != nil || !pgf.File.Package.IsValid() {
-				return nil // need a valid header
-			}
 			return s.load(ctx, false, fileLoadScope(file.URI()))
 		})
 	}
@@ -1670,7 +1632,7 @@ func (s *snapshot) reloadOrphanedOpenFiles(ctx context.Context) error {
 		// TODO(rfindley): instead of locking here, we should have load return the
 		// metadata graph that resulted from loading.
 		uri := file.URI()
-		if len(s.meta.ids) == 0 {
+		if len(s.meta.ids[uri]) == 0 {
 			s.unloadableFiles[uri] = struct{}{}
 		}
 	}
@@ -1688,22 +1650,6 @@ func (s *snapshot) reloadOrphanedOpenFiles(ctx context.Context) error {
 // reloadOrphanedFiles. The latter does not include files with
 // command-line-arguments packages.
 func (s *snapshot) OrphanedFileDiagnostics(ctx context.Context) (map[span.URI]*source.Diagnostic, error) {
-	// Orphaned file diagnostics are queried from code actions to produce
-	// quick-fixes (and may be queried many times, once for each file).
-	//
-	// Because they are non-trivial to compute, record them optimistically to
-	// avoid most redundant work.
-	//
-	// This is a hacky workaround: in the future we should avoid recomputing
-	// anything when codeActions provide a diagnostic: simply read the published
-	// diagnostic, if it exists.
-	s.mu.Lock()
-	existing := s.orphanedFileDiagnostics
-	s.mu.Unlock()
-	if existing != nil {
-		return existing, nil
-	}
-
 	if err := s.awaitLoaded(ctx); err != nil {
 		return nil, err
 	}
@@ -1880,8 +1826,7 @@ https://github.com/golang/tools/blob/master/gopls/doc/settings.md#buildflags-str
 		}
 
 		if msg != "" {
-			// Only report diagnostics if we detect an actual exclusion.
-			diagnostics[fh.URI()] = &source.Diagnostic{
+			d := &source.Diagnostic{
 				URI:            fh.URI(),
 				Range:          rng,
 				Severity:       protocol.SeverityWarning,
@@ -1889,15 +1834,14 @@ https://github.com/golang/tools/blob/master/gopls/doc/settings.md#buildflags-str
 				Message:        msg,
 				SuggestedFixes: suggestedFixes,
 			}
+			if ok := source.BundleQuickFixes(d); !ok {
+				bug.Reportf("failed to bundle quick fixes for %v", d)
+			}
+			// Only report diagnostics if we detect an actual exclusion.
+			diagnostics[fh.URI()] = d
 		}
 	}
-
-	s.mu.Lock()
-	defer s.mu.Unlock()
-	if s.orphanedFileDiagnostics == nil { // another thread may have won the race
-		s.orphanedFileDiagnostics = diagnostics
-	}
-	return s.orphanedFileDiagnostics, nil
+	return diagnostics, nil
 }
 
 // TODO(golang/go#53756): this function needs to consider more than just the
@@ -2009,7 +1953,6 @@ func (s *snapshot) clone(ctx, bgCtx context.Context, changes map[span.URI]*fileC
 		packages:             s.packages.Clone(),
 		activePackages:       s.activePackages.Clone(),
 		files:                s.files.Clone(),
-		parseCache:           s.parseCache,
 		symbolizeHandles:     s.symbolizeHandles.Clone(),
 		workspacePackages:    make(map[PackageID]PackagePath, len(s.workspacePackages)),
 		unloadableFiles:      make(map[span.URI]struct{}, len(s.unloadableFiles)),
@@ -2158,6 +2101,10 @@ func (s *snapshot) clone(ctx, bgCtx context.Context, changes map[span.URI]*fileC
 		}
 
 		// Make sure to remove the changed file from the unloadable set.
+		//
+		// TODO(rfindley): this also looks wrong, as typing in an unloadable file
+		// will result in repeated reloads. We should only delete if metadata
+		// changed.
 		delete(result.unloadableFiles, uri)
 	}
 
@@ -2452,8 +2399,8 @@ func metadataChanges(ctx context.Context, lockedSnapshot *snapshot, oldFH, newFH
 
 	fset := token.NewFileSet()
 	// Parse headers to compare package names and imports.
-	oldHeads, oldErr := lockedSnapshot.parseCache.parseFiles(ctx, fset, source.ParseHeader, oldFH)
-	newHeads, newErr := lockedSnapshot.parseCache.parseFiles(ctx, fset, source.ParseHeader, newFH)
+	oldHeads, oldErr := lockedSnapshot.view.parseCache.parseFiles(ctx, fset, source.ParseHeader, false, oldFH)
+	newHeads, newErr := lockedSnapshot.view.parseCache.parseFiles(ctx, fset, source.ParseHeader, false, newFH)
 
 	if oldErr != nil || newErr != nil {
 		// TODO(rfindley): we can get here if newFH does not exist. There is
@@ -2504,8 +2451,8 @@ func metadataChanges(ctx context.Context, lockedSnapshot *snapshot, oldFH, newFH
 	// Note: if this affects performance we can probably avoid parsing in the
 	// common case by first scanning the source for potential comments.
 	if !invalidate {
-		origFulls, oldErr := lockedSnapshot.parseCache.parseFiles(ctx, fset, source.ParseFull, oldFH)
-		newFulls, newErr := lockedSnapshot.parseCache.parseFiles(ctx, fset, source.ParseFull, newFH)
+		origFulls, oldErr := lockedSnapshot.view.parseCache.parseFiles(ctx, fset, source.ParseFull, false, oldFH)
+		newFulls, newErr := lockedSnapshot.view.parseCache.parseFiles(ctx, fset, source.ParseFull, false, newFH)
 		if oldErr == nil && newErr == nil {
 			invalidate = magicCommentsChanged(origFulls[0].File, newFulls[0].File)
 		} else {
@@ -2590,7 +2537,7 @@ func (s *snapshot) BuiltinFile(ctx context.Context) (*source.ParsedGoFile, error
 	// For the builtin file only, we need syntactic object resolution
 	// (since we can't type check).
 	mode := source.ParseFull &^ source.SkipObjectResolution
-	pgfs, err := s.parseCache.parseFiles(ctx, token.NewFileSet(), mode, fh)
+	pgfs, err := s.view.parseCache.parseFiles(ctx, token.NewFileSet(), mode, false, fh)
 	if err != nil {
 		return nil, err
 	}
